local g3d = require("g3d")

function generate_line(start_x, start_y, start_z, end_x, end_y, end_z, width)

    local line_direction = {end_x - start_x, end_y - start_y, end_z - start_z}
    local line_length = math.sqrt(line_direction[1]^2 + line_direction[2]^2 + line_direction[3]^2)

    line_direction[1] = line_direction[1] / line_length
    line_direction[2] = line_direction[2] / line_length
    line_direction[3] = line_direction[3] / line_length

    local offset = {width * line_direction[2], -width * line_direction[1], 0}

    local vertices = {}

    for i = 0, line_length, width do
        local x1, y1, z1 = start_x + i * line_direction[1] - offset[1], start_y + i * line_direction[2] - offset[2], start_z + i * line_direction[3] - offset[3]
        local x2, y2, z2 = start_x + i * line_direction[1] + offset[1], start_y + i * line_direction[2] + offset[2], start_z + i * line_direction[3] + offset[3]
        local x3, y3, z3 = start_x + (i + width) * line_direction[1] + offset[1], start_y + (i + width) * line_direction[2] + offset[2], start_z + (i + width) * line_direction[3] + offset[3]
        local x4, y4, z4 = start_x + (i + width) * line_direction[1] - offset[1], start_y + (i + width) * line_direction[2] - offset[2], start_z + (i + width) * line_direction[3] - offset[3]

        table.insert(vertices, {x1, y1, z1})
        table.insert(vertices, {x2, y2, z2})
        table.insert(vertices, {x3, y3, z3})
        table.insert(vertices, {x1, y1, z1})
        table.insert(vertices, {x3, y3, z3})
        table.insert(vertices, {x4, y4, z4})
    end

    return vertices

end

function generate_cylinder(height, radius, nb_segments)

    local vertices = {}
    local angle_step = 2 * math.pi / nb_segments
    local angle = 0

    local base_z = -height / 2
    local tip_z = height / 2

    for i = 1, nb_segments do

        local x1 = radius * math.cos(angle)
        local y1 = radius * math.sin(angle)
        local x2 = radius * math.cos(angle + angle_step)
        local y2 = radius * math.sin(angle + angle_step)

        -- Base triangle
        table.insert(vertices, {0,0,base_z})
        table.insert(vertices, {x1,y1,base_z})
        table.insert(vertices, {x2,y2,base_z})

        -- Tip triangle
        table.insert(vertices, {0,0,tip_z})
        table.insert(vertices, {x1,y1,tip_z})
        table.insert(vertices, {x2,y2,tip_z})

        -- Side triangle
        table.insert(vertices, {x1,y1,base_z})
        table.insert(vertices, {x1,y1,tip_z})
        table.insert(vertices, {x2,y2,base_z})

        table.insert(vertices, {x2,y2,tip_z})
        table.insert(vertices, {x2,y2,base_z})
        table.insert(vertices, {x1,y1,tip_z})

        angle = angle + angle_step

    end

    return vertices

end


function love.load()

    PlaneLine = g3d.newModel(
        generate_line(0,-1,0, 0,1,0, .1), nil,
        {10,0,0}, {0,math.pi/2,0}, {1,10,1}
    )

    CylinderLine = g3d.newModel(
        generate_cylinder(10, .05, 256), nil,
        {10,0,5}, {math.pi/2,math.pi/2,0}
    )

end

function love.update(dt)
    g3d.camera.firstPersonMovement(dt)
end

function love.mousemoved(x,y,dx,dy)
    g3d.camera.firstPersonLook(dx,dy)
end

function love.draw()

    love.graphics.setColor(1,1,1)
    PlaneLine:draw()

    love.graphics.setColor(1,1,0)
    CylinderLine:draw()

end


